#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Copyright (c) 2020
# @Author :  GitOPEN
# @Email  :  gitopen@gmail.com
# @Date   :  2020-10-29 13:38:36
# @Description :  有关文件操作的相关工具函数

__version__ = "1.02.10"

import asyncio
import json
import pickle
import time
from pathlib import Path
from typing import Callable, Iterable, List, Optional, Union

from gitopenlib.utils import basics as gb
import pandas as pd
from pandas import DataFrame


def df_to_json(
    df: DataFrame,
    path: str,
    encoding="utf-8",
    force_ascii=False,
):
    """把 DataFrame 保存为 json 文件。

    Args:
        df: DataFrame。
        path: 保存路径。
        orient: 保存格式，参考 `pandas.DataFrame.to_json` 的参数说明。
    """
    if_path_exist_then_backup(path)
    with open(path, "w+", encoding=encoding) as f:
        df.to_json(f, force_ascii=force_ascii)


def df_from_json(
    path: str,
    encoding="utf-8",
) -> DataFrame:
    """将json数据读取为 `pandas.DataFrame` 。"""
    return pd.read_json(path, encoding=encoding)


def save_df(
    df: DataFrame,
    path: str,
    format: str = "xlsx|json",
    encoding="utf-8-sig",
    backup: bool = True,
) -> None:
    """把 pandas 的 Dataframe 保存为 xlsx(csv) 、json 文件。

    注：该方法使用pickle在3.7和3.8两个版本中存在兼容问题，请注意这一点。

    Args:
        df: `DataFrame`数据。
        path: 文件路径，给出一个文件路径即可。
        format: 可选格式包括`xlsx|json`、`csv|json`、`json`、`xlsx`、`csv`；默认为`xlsx|json`。
        encoding: `xlsx(csv)`的编码格式，`utf-8-sig`方便windows系统打开它时不乱码；`json`的编码格式为`utf-8`。
        backup: `True`表示文件存在时进行备份(推荐)，`False`表示不备份。
    Returns:
        None.
    """

    path = path.replace(".xlsx", "").replace(".csv", "").replace(".json", "")
    path = path + ".{}"

    def to_file(ft):
        file_path = path.format(ft)
        if backup:
            if_path_exist_then_backup(file_path)
        if ft == "xlsx":
            df.to_excel(file_path, encoding=encoding)
        if ft == "csv":
            df.to_csv(file_path, encoding=encoding)
        if ft == "json":
            df_to_json(df, file_path)

    fts = format.split("|")
    for ft in fts:
        to_file(ft)


def save_pkl(obj, path: str, backup: bool = True):
    """序列化对象并保存到磁盘。

    Args:
        object: 序列化对象
        path: 目标路径
        backup: `True`表示文件存在时进行备份(推荐)，`False`表示不备份。

    Returns:
        bool: 保存成功，返回 True，保存失败，返回 False，抛出异常。
    """
    if backup:
        if_path_exist_then_backup(path)
    file = open(path, mode="wb")
    pickle.dump(obj, file)
    file.close()


def read_pkl(path: str):
    """从磁盘读取序列化对象"""
    return pickle.load(open(path, "rb"))


def check_path(pathes: Union[str, List[str]]):
    """检查路径是否存在，不存在则创建。创建时会自动创建父目录。"""
    if isinstance(pathes, str):
        pathes = [pathes]
    for path in pathes:
        path = Path(path)
        if not path.exists():
            path.mkdir(parents=True, exist_ok=True)


def if_path_exist_then_backup(pathes: Union[str, List[str]]) -> bool:
    """
    检查路径是否存在，如路径存在，则备份。

    Args:
        pathes: 路径，可以是单个字符串或者字符串的列表。

    Returns:
        bool: 如果有文件被备份了，则为True，否则为False
    """
    if isinstance(pathes, str):
        pathes = [pathes]

    has_backup_files = False
    for path in pathes:
        path = Path(path)
        if path.exists():
            path.rename(
                path.with_suffix(
                    f'.{str(time.strftime("%Y%m%d_%H%M%S",time.localtime()))}{path.suffix}'
                )
            )
            has_backup_files = True
    return has_backup_files


def new_dirs(dir_paths: Union[str, List[str]]) -> List[str]:
    """
    初始化文件夹，检验文件夹的存在状态，并返回准备好的文件夹路径。

    Args:
        dir_paths: 字符串类型的全路径，可以为单个路径，也可以放入列表中，批量创建。

    Returns:
        包含创建的文件夹路径字符串的list
    """
    if isinstance(dir_paths, str):
        dir_paths = [dir_paths]

    for dir in dir_paths:
        dir_path = Path(dir)
        dir_path.mkdir(parents=True, exist_ok=True)

    return dir_paths


def get_paths_from_dir(
    dirs: Union[str, List[str]],
    types: Optional[Union[str, List[str]]] = None,
    recusive: bool = False,
) -> List[str]:
    """
    从指定目录下获取所有指定扩展名文件的路径

    Args:
        dirs: 文件夹路径（绝对路径），单个用str表示，多个用list
        types: 指定文件的扩展名，单个用str表示，多个用list，默认为None，表示所有类型文件
        recusive: 默认为False，不递归子文件夹

    Returns:
        文件路径字符串列表
    """

    if isinstance(dirs, str):
        dirs = [dirs]
    if types is not None:
        if isinstance(types, str):
            types = [types]
        rule = "**/*." if recusive else "*."
    else:
        rule = "**/*" if recusive else "*"
        types = [""]

    result = []
    for d in dirs:
        path = Path(d).resolve()
        for t in types:
            res = path.glob(rule + t)
            result.extend(list(res))

    return [str(item) for item in result]


def file_writer(
    lines: Union[str, Iterable[str]],
    dir_path: Union[str, Path],
    file_name: str = "file_writer.txt",
    mode: str = "a+",
    separator: str = "\n",
    encoding: str = "utf-8",
    backup: bool = True,
) -> None:
    """向文件中写内容

    Args:
        lines: 可以是单个字符串或者字符串的列表
        dir_path: 文件的绝对路径
        file_name: 文件名称，有默认值
        mode: 写文件的的模式，默认为 a+
        separator: 每一行末尾的分隔符，有默认值
        encoding: 文件的编码格式，默认为utf-8
        backup: 如果文件已经存在，是否对原文件进行备份，默认为True；
            如果为False，当文件已经存在的情况下，会追加到存在的文件中。
    """
    dir_path: Path = Path(dir_path) if isinstance(dir_path, str) else dir_path

    if not dir_path.exists():
        print(
            "## Warning: The dir path you submit is not exist, "
            + "it will be created automatically."
        )
        dir_path.mkdir(parents=True, exist_ok=True)

    file_path = dir_path / file_name

    if backup:
        if file_path.exists():
            file_path.rename(
                file_path.with_suffix(
                    f'.{str(time.strftime("%Y%m%d_%H%M%S",time.localtime()))}{file_path.suffix}'
                )
            )

    if isinstance(lines, str):
        lines = [lines]

    with open(file_path, mode=mode, encoding=encoding) as file:
        for line in lines:
            if separator != "":
                file.write(f"{line}{separator}")
            else:
                file.write(line)


def read_txt_by_page(
    file_path: str,
    parse_func: Callable[[list], None],
    page_size: int = 100,
    encoding: str = "utf-8",
) -> None:
    """
    从文本文件中分页读取内容

    Args:
        file_path: 文件路径
        parse_func: 每一页数据处理函数，必须定义
        page_size: 每一页数据量
        encoding: 文本文件的编码格式
    """
    with open(file=file_path, encoding=encoding, mode="r") as file:
        data = list()
        curr_page_id = 0
        start_time = time.time()
        total_time = 0.0
        print("## read_txt_by_page is starting parse the data...")
        for line in file:
            data.append(line.strip())
            if len(data) == page_size:
                parse_func(data)
                data.clear()
                end_time = time.time()
                cost_time = float(end_time - start_time)
                total_time += cost_time
                start_time = end_time
                print(f"## -{curr_page_id}- page parsed done...{[cost_time]}s")
                curr_page_id += 1

            pass
        if len(data) > 0:
            parse_func(data)
            end_time = time.time()
            cost_time = float(end_time - start_time)
            total_time += cost_time
            start_time = end_time
            print(f"## -{curr_page_id}- page parsed done...{[cost_time]}s")
            curr_page_id += 1
    print(f"## All done. Total pages: {curr_page_id}. Elapsed time: {total_time}s")


def read_txt_by_page_asyncio(
    file_path: str,
    parse_func: Callable[[list], None],
    page_size: int = 100,
    encoding: str = "utf-8",
    open_async: bool = False,
    slave_num: int = 4,
) -> None:
    """
    从文本文件中分页读取内容，具备异步io处理功能

    Args:
        file_path: 文件路径
        parse_func: 每一页数据处理函数，必须定义
        page_size: 每一页数据量
        encoding: 文本文件的编码格式
        open_async: 是否开启异步io处理数据�������������默认不开启
        slave_num: �������行�������务的���程数目，默认为4
    """

    async def parse_(loop, chunk):
        run_func = lambda: parse_func(chunk)
        await loop.run_in_executor(None, run_func)

    with open(file=file_path, encoding=encoding, mode="r") as file:
        data = list()
        curr_page_id = 0
        start_time = time.time()
        total_time = 0.0
        print("## read_txt_by_page is starting parse the data...")
        for line in file:
            data.append(line.strip())
            if len(data) == page_size:
                if open_async:
                    loop = asyncio.get_event_loop()
                    chunks = gb.chunks(data, slave_num)
                    loop.run_until_complete(
                        asyncio.gather(*[parse_(loop, chunk) for chunk in chunks])
                    )
                    chunks.clear()
                else:
                    parse_func(data)
                # parse_func(data)
                data.clear()
                end_time = time.time()
                cost_time = float(end_time - start_time)
                total_time += cost_time
                start_time = end_time
                print(f"## -{curr_page_id}- page parsed done...{[cost_time]}s")
                curr_page_id += 1

            pass
        if len(data) > 0:
            if open_async:
                loop = asyncio.get_event_loop()
                chunks = gb.chunks(data, slave_num)
                loop.run_until_complete(
                    asyncio.gather(*[parse_(loop, chunk) for chunk in chunks])
                )
                chunks.clear()
            else:
                parse_func(data)
            # parse_func(data)
            end_time = time.time()
            cost_time = float(end_time - start_time)
            total_time += cost_time
            start_time = end_time
            print(f"## -{curr_page_id}- page parsed done...{[cost_time]}s")
            curr_page_id += 1
    print(f"## All done.Total pages: {curr_page_id}.Cost time: {total_time}s")


def read_content(
    file_path: Union[str, Path],
    encoding: str = "utf-8",
) -> List[str]:
    """
    从文本文件中读取内容，将内容转换为list，list的元素为每行的字符串

    Args:
        file_path : 文件路径
        encoding : 文件的编码方式

    Returns:
        每行字符串组成的list
    """
    if isinstance(file_path, str):
        file_path = Path(file_path)
    lines = file_path.read_text(encoding=encoding).split("\n")
    result = gb.remove_0_str([line.strip() for line in lines])
    return result


def read_contents(
    file_pathes: List,
    encoding: str = "utf-8",
) -> List[str]:
    """
    从[多个]文本文件中读取内容，将[所有]内容转换为list，list的元素为每行的字符串

    Args:
        file_pathes : 文件路径列表
        encoding : 文件的编码方式

    Returns:
        每行字符串组成的list
    """
    result = []
    for path in file_pathes:
        result.extend(read_content(path, encoding))
    return result


def read_jsons(
    file_path: Union[str, Path],
    encoding: str = "utf-8",
) -> List[dict]:
    """
    从存放json的文本文件中读取内容，并转化为dict组成的list

    Args:
        file_path: 文件路径
        encoding: 文件的编码方式

    Returns:
        dict组成的list
    """
    if isinstance(file_path, str):
        file_path = Path(file_path)
    lines = file_path.read_text(encoding=encoding).split("\n")
    result = gb.remove_0_str([line.strip() for line in lines])

    return [json.loads(item) for item in result]



#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Copyright (c) 2021
# @Author :  GitOPEN
# @Email  :  gitopen@gmail.com
# @Date   :  2021-03-18 10:54:33
# @Description :  一些常用的有关 机器学习、深度学习 的通用函数

__version__ = "0.2.7"


import math
from typing import List
import matplotlib.pyplot as plt
import numpy as np
import itertools
from sklearn.metrics import confusion_matrix as sk_cm
from gitopenlib.utils import files as gf


def confusion_matrix(
    y_true,
    y_pred,
    classes=None,
    normalize=False,
    decimals=3,
    figsize=(10, 10),
    fontsize=14,
    textsize=12,
    cmap=plt.cm.Blues,
    svg="save",
    svgpath="./confusion_matrix.svg",
):
    """
    打印出优化版的混淆矩阵及其可视化。

    Args:
        y_true: 真实标签。
        y_pred: 预测结果标签。
        classes: 类别名（标签名）。
        normalize: 是否标准化。
        decimals: 标准化数值小数位数。
        figsize: 图片大小。
        fontsize: 图片上的字体大小。
        textsize: 热力图中数值的字体大小。
        cmap: 图片的配色方案。
        svg: svg的现实方法，'show'表示直接显示；'save'表示只保存不显示；'both'表示既显示又保存。
        svgpath: 文件路径。
    """
    cm = sk_cm(y_true, y_pred)
    if normalize:
        print("Normalized confusion matrix")
        cm_norm = cm.astype("float32") / cm.sum(axis=1)[:, np.newaxis]
        print(cm_norm)
    else:
        print("Confusion matrix, without normalization")
        print(cm)

    plt.figure(figsize=figsize)
    plt.imshow(cm, interpolation="nearest", cmap=cmap)
    plt.colorbar()

    if not classes:
        classes = list(range(cm.shape[0]))
    tick_marks = np.arange(len(classes))
    plt.xticks(tick_marks, classes, rotation=0 if classes else 45)
    plt.yticks(tick_marks, classes)

    thresh = cm.max() / 2.0
    for i, j in itertools.product(range(cm.shape[0]), range(cm.shape[1])):
        text = (
            "{}\n({})".format(cm[i, j], round(cm_norm[i, j], decimals))
            if normalize
            else cm[i, j]
        )
        plt.text(
            j,
            i,
            text,
            horizontalalignment="center",
            verticalalignment="center",
            color="white" if cm[i, j] > thresh else "black",
            size=textsize,
        )

    plt.title("Confusion matrix", fontsize=fontsize)
    plt.ylabel("True label", fontsize=fontsize)
    plt.xlabel("Predicted label", fontsize=fontsize)

    plt.tight_layout()

    if svg == "save":
        gf.if_path_exist_then_backup(svgpath)
        plt.savefig(svgpath)
    elif svg == "both":
        gf.if_path_exist_then_backup(svgpath)
        plt.savefig(svgpath)
        plt.show()
    else:
        plt.show()


def softmax(data: list, decimal: int = None) -> List[float]:
    """
    归一化指数函数softmax，将一个任意实数的K维向量，压缩到另一个K维向量中，使得每一个元素
    的范围都在(0,1)之间，并且所有元素的和为1。
    """
    z_exp = [math.exp(i) for i in data]
    sum_z_exp = sum(z_exp)

    softmax = [
        round(i / sum_z_exp, decimal) if decimal is not None else (i / sum_z_exp)
        for i in z_exp
    ]
    return softmax



#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Copyright (c) 2022
# @Author :  GitOPEN
# @Email  :  gitopen@gmail.com
# @Date   :  2022-09-17 11:31:23
# @Description :  Some useful things about debug.

__version__ = "0.2.01"

import sys
import traceback


def catch_except_run(func, args):
    """运行指定函数，并捕获异常

    Args:
        func (function): 指定函数。
        args (tuple): 函数参数。

    Returns:
        function: 若未报错，返回函数执行结果；若报错，返回None。

    """
    try:
        return func(*args)
    except Exception:
        traceback.print_exc()
        return None


def exit(status: int = 0):
    sys.exit(status)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

# Copyright (c) 2022
# @Author :  GitOPEN
# @Email  :  gitopen@gmail.com
# @Date   :  2022-08-13 22:08:47
# @Description :  Get the ip address.


__version__ = "0.0.1"


import requests
from requests import HTTPError
from lxml import etree
from gitopenlib.utils import basics as gb
import socket


def get_text(url):
    header = {
        "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.9999.0 Safari/537.36"
    }
    resp = requests.get(url, headers=header)
    if resp.status_code != 200:
        raise HTTPError("Please check your network!")

    return resp.text


def get_wan(html):
    html = etree.fromstring(html, parser=etree.HTMLParser())
    ipv4 = html.xpath("//p[1]/a[1]/text()")[0]

    info = html.xpath("//p[1]/text()")[-1].split("来自：")[-1].split()
    location, service = info[0], info[1]

    result = {"wan_ip": ipv4, "location": location, "service": service}

    return result


def get_lan():
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        ip = s.getsockname()[0]
    finally:
        s.close()
        return ip


def get_ip():
    url1 = "https://2022.ip138.com/"
    html = get_text(url1)
    wan_res = get_wan(html)

    lan_ip = get_lan()
    wan_res["lan_ip"] = lan_ip
    gb.printj(wan_res, sort_keys=True)
    return wan_res


#  if __name__ == "__main__":
#      get_ip()
